/* version : 1.4 */

Sprite_Enemy.prototype.mainSprite = function() {
	return this._mainSprite;
};

Sprite_Enemy.prototype.createMainSprite = function() {
	this._mainSprite = new Sprite();
	this._mainSprite.anchor.x = 0.5;
	this._mainSprite.anchor.y = 1;
	this.addChild(this._mainSprite);
};

Sprite_Enemy.prototype.initMembers = function() {
	Sprite_Battler.prototype.initMembers.call(this);
	this._enemy = null;
	this._appeared = false;
	this._battlerName = "";
	this._battlerHue = 0;
	this._effectType = null;
	this._effectDuration = 0;
	this._shake = 0;
	this.createStateIconSprite();
	
	//add cj mainSprite
	this.createMainSprite();
	this.createShadowSprite();
	this._motionRefresh = false;
	this._motionCount = 0;
	this._motion = null;
};

Sprite_Enemy.prototype.createStateIconSprite = function() {
	this._stateIconSprite = new Sprite_StateIcon();
	this.addChild(this._stateIconSprite);
};

Sprite_Enemy.prototype.updateFrame = function() {
	Sprite_Battler.prototype.updateFrame.call(this);
	//add bitmap update
	const bitmap = this._mainSprite.bitmap;
	
	if (this._effectType === "bossCollapse") {
		this.setFrame(0, 0, this.bitmap.width, this._effectDuration);
	} else if (bitmap) {
		const motionIndex = this._motion ? this._motion.index : 0;
		const pattern = this._pattern < 3 ? this._pattern : 1;
		const cw = bitmap.width / 9;
		const ch = bitmap.height / 6;
		const cx = Math.floor(motionIndex / 6) * 3 + pattern;
		const cy = motionIndex % 6;
		this._mainSprite.setFrame(cx * cw, cy * ch, cw, ch);
		this.setFrame(0, 0, 0, 0);
	} else {
		this.setFrame(0, 0, this.bitmap.width, this.bitmap.height);
	}
};

Sprite_Enemy.prototype.updateBitmap = function() {
	Sprite_Battler.prototype.updateBitmap.call(this);
	const name = this._enemy.battlerName();
	const hue = this._enemy.battlerHue();
	
	if (this._battlerName !== name || this._battlerHue !== hue) {
		this._battlerName = name;
		this._battlerHue = hue;
		this.loadBitmap(name);
		this.setHue(hue);
		this.initVisibility();
		
		//add cj bitmap detail
		this._mainSprite.bitmap = ImageManager.loadSvEnemy(name);
	}
};

Sprite_Enemy.MOTIONS = {
	walk: { index: 0, loop: true },			//站立动画
	wait: { index: 1, loop: true },			//战斗准备姿势
	chant: { index: 2, loop: true },		//吟唱
	guard: { index: 3, loop: true },		//防御
	damage: { index: 4, loop: false },		//受到伤害时
	evade: { index: 5, loop: false },		//闪避动作
	thrust: { index: 6, loop: false },		//攻击
	swing: { index: 7, loop: false },
	missile: { index: 8, loop: false },
	skill: { index: 9, loop: false },
	spell: { index: 10, loop: false },
	item: { index: 11, loop: false },
	escape: { index: 12, loop: true },
	victory: { index: 13, loop: true },
	dying: { index: 14, loop: true },		//跪撑
	abnormal: { index: 15, loop: true },	//跪撑
	sleep: { index: 16, loop: true },		//跪下
	dead: { index: 17, loop: true }			//扑街
};

Sprite_Enemy.prototype.setBattler = function(battler) {
	Sprite_Battler.prototype.setBattler.call(this, battler);
	this._enemy = battler;
	this.setHome(battler.screenX(), battler.screenY());
	this.startMotion("walk");				//see Sprite_Actor.MOTIONS
	this._stateIconSprite.setup(battler);
};

Sprite_Enemy.prototype.createShadowSprite = function() {
	this._shadowSprite = new Sprite();
	this._shadowSprite.bitmap = ImageManager.loadSystem("Shadow2");
	this._shadowSprite.anchor.x = 0.44;
	this._shadowSprite.anchor.y = 0.5;
	this._shadowSprite.y = -2;
	this.addChild(this._shadowSprite);
};

/* to make enemy anim actions play */

Sprite_Enemy.prototype.setupMotion = function() {
	if (this._enemy.isMotionRequested()) {		//always false
		this.startMotion(this._enemy.motionType());
		this._enemy.clearMotion();
	}
};

Sprite_Enemy.prototype.startMotion = function(motionType) {
	const newMotion = Sprite_Enemy.MOTIONS[motionType];
	if (this._motion !== newMotion) {
		this._motion = newMotion;
		this._motionCount = 0;
		this._pattern = 0;
	}
};

//站立动画、攻击动画、重伤动画（HP低于25%） 会更新
//被攻击动画、闪避动画，不做
//死亡动画，在死亡判定中单独实现
Sprite_Enemy.prototype.updateMotion = function() {
	this.setupMotion();
	if (this._enemy.isMotionRefreshRequested()) {	// be hit, dead ;anim not support
		this.refreshMotion();
		this._enemy.clearMotion();
	}
	this.updateMotionCount();
};

Sprite_Enemy.prototype.updateMotionCount = function() {
	if (this._motion && ++this._motionCount >= this.motionSpeed()) {
		if (this._motion.loop) {
			this._pattern = (this._pattern + 1) % 4;
		} else if (this._pattern < 2) {
			this._pattern++;
		} else {
			this.refreshMotion();
		}
		this._motionCount = 0;
	}
};

Sprite_Enemy.prototype.refreshMotion = function() {
	const enemy = this._enemy;
	if (enemy) {
		const stateMotion = enemy.stateMotionIndex();
		if (enemy.isInputting() || enemy.isActing()) {
			this.startMotion("thrust");
		} else if (stateMotion === 3) {
			this.startMotion("dead");
		} else if (stateMotion === 2) {
			this.startMotion("sleep");
		} else if (enemy.isChanting()) {
			this.startMotion("chant");
		} else if (enemy.isGuard() || enemy.isGuardWaiting()) {
			this.startMotion("guard");
		} else if (stateMotion === 1) {
			this.startMotion("abnormal");
		} else if (enemy.isDying()) {
			this.startMotion("dying");
		} else if (enemy.isUndecided()) {
			this.startMotion("walk");
		} else {
			this.startMotion("walk");
		}
	}
};

Sprite_Enemy.prototype.motionSpeed = function() {
	return 12;
};

Sprite_Enemy.prototype.update = function() {
	Sprite_Battler.prototype.update.call(this);
	if (this._enemy) {
		this.updateEffect();
		this.updateStateSprite();
		this.updateMotion();
	}
};


Sprite.prototype.setFrame = function(x, y, width, height) {
	this._refreshFrame = false;
	const frame = this._frame;
	if (
		x !== frame.x ||
		y !== frame.y ||
		width !== frame.width ||
		height !== frame.height
	) {
		frame.x = x;
		frame.y = y;
		frame.width = width;
		frame.height = height;
		this._refresh();
	}
};

Sprite.prototype._refresh = function() {
	const texture = this.texture;
	const frameX = Math.floor(this._frame.x);
	const frameY = Math.floor(this._frame.y);
	const frameW = Math.floor(this._frame.width);
	const frameH = Math.floor(this._frame.height);
	const baseTexture = this._bitmap ? this._bitmap.baseTexture : null;
	const baseTextureW = baseTexture ? baseTexture.width : 0;
	const baseTextureH = baseTexture ? baseTexture.height : 0;
	const realX = frameX.clamp(0, baseTextureW);
	const realY = frameY.clamp(0, baseTextureH);
	const realW = (frameW - realX + frameX).clamp(0, baseTextureW - realX);
	const realH = (frameH - realY + frameY).clamp(0, baseTextureH - realY);
	const frame = new Rectangle(realX, realY, realW, realH);
	if (texture) {
		this.pivot.x = frameX - realX;
		this.pivot.y = frameY - realY;
		if (baseTexture) {
			texture.baseTexture = baseTexture;
			try {
				texture.frame = frame;
			} catch (e) {
				texture.frame = new Rectangle();
			}
		}
		texture._updateID++;
	}
};

Bitmap.prototype.blt = function(source, sx, sy, sw, sh, dx, dy, dw, dh) {
	dw = dw || sw;
	dh = dh || sh;
	try {
		const image = source._canvas || source._image;
		this.context.globalCompositeOperation = "source-over";
		this.context.drawImage(image, sx, sy, sw, sh, dx, dy, dw, dh);
		this._baseTexture.update();
	} catch (e) {
		//
	}
};

Game_BattlerBase.prototype.addNewState = function(stateId) {
	if (stateId === this.deathStateId()) {
		this.die();
		
		//cj add enemy dead anim
		if(this.hasOwnProperty('_enemyId')){
			this.requestMotionRefresh();
		}
	}
	const restricted = this.isRestricted();
	this._states.push(stateId);
	this.sortStates();
	if (!restricted && this.isRestricted()) {
		this.onRestrict();
	}
};