/* =========================
	mobile screen	
========================= */

document.styleSheets[0].insertRule('@media screen and (orientation: portrait) {\
	#gameCanvas,#gameVideo,#mobileControls{\
		width:100vmax;\
		height:100vmin;\
		\
		transform-origin:0px 0px;\
		-ms-transform-origin:0px 0px;\
		-moz-transform-origin:0px 0px;\
		-webkit-transform-origin:0px 0px;\
		-o-transform-origin:0px 0px;\
		\
		transform:rotate(90deg) translate(0, -100vmin);\
		-ms-transform:rotate(90deg) translate(0, -100vmin);\
		-moz-transform:rotate(90deg) translate(0, -100vmin);\
		-webkit-transform:rotate(90deg) translate(0, -100vmin);\
		-o-transform:rotate(90deg) translate(0, -100vmin);\
	}\
}', document.styleSheets[0].cssRules.length);

Graphics._updateAllElements = function() {
	this._updateRealScale();
	this._updateErrorPrinter();
	this._updateCanvas();
	this._updateVideo();
	
	//add cj mobile panel center s
	this._updatePanel();
};

Graphics._updateRealScale = function() {
	let h, v;
	if (this._stretchEnabled && this._width > 0 && this._height > 0) {
		if (this._stretchWidth() > this._stretchHeight()) {
			h = this._stretchWidth() / this._width;
			v = this._stretchHeight() / this._height;
		} else {
			h = this._stretchHeight() / this._width;
			v = this._stretchWidth() / this._height;
		}
		this._realScale = Math.min(h, v);
		window.scrollTo(0, 0);
	} else {
		this._realScale = this._defaultScale;
	}
};

Graphics._updateVideo = function() {
	let width = this._width * this._realScale;
	let height = this._height * this._realScale;
	let top;
	if (this._stretchWidth() > this._stretchHeight()) {
		left = -1;
		top = -1;
	} else {
		left = -(this._stretchWidth() - height) / 2;
		top = (this._stretchHeight() - width) / 2;
	}
	Video.resize(width, height, left, top);
};

Video.resize = function(width, height, left, top) {
	if (this._element) {
		if (top < 0) {
			this._element.style.margin = 'auto';
			this._element.style.top = 0;
			this._element.style.left = 0;
			this._element.style.right = 0;
			this._element.style.bottom = 0;
		} else {
			this._element.style.margin = 0;
			this._element.style.top = top + "px";
			this._element.style.left = left + "px";
			this._element.style.right = '';
			this._element.style.bottom = '';
		}
		this._element.style.width = width + "px";
		this._element.style.height = height + "px";
	}
};

Graphics._centerElement = function(element) {
	let width, height;
	width = element.width * this._realScale;
	height = element.height * this._realScale;
	
	element.style.position = "absolute";
	if (this._stretchWidth() > this._stretchHeight()) {
		element.style.margin = 'auto';
		element.style.top = 0;
		element.style.left = 0;
		element.style.right = 0;
		element.style.bottom = 0;
	} else {
		element.style.margin = 0;
		element.style.top = ((this._stretchHeight() - width) / 2) + "px";
		element.style.left = -((this._stretchWidth() - height) / 2 + 1) + "px";
		element.style.right = '';
		element.style.bottom = '';
	}
	
	element.style.width = width + "px";
	element.style.height = height + "px";
};

Utils.isLocal = function() {
	//return window.location.href.startsWith("file:");
	return true;
};

TouchInput._onTouchStart = function(event) {
	let x, y;
	for (let touch of event.changedTouches) {
		if (Graphics._stretchWidth() > Graphics._stretchHeight()) {
			x = Graphics.pageToCanvasX(touch.pageX);
			y = Graphics.pageToCanvasY(touch.pageY);
		} else {
			y = Graphics._stretchWidth() / Graphics._realScale - Graphics.pageToCanvasX(touch.pageX);
			x = Graphics.pageToCanvasY(touch.pageY);
		}
		
		if (Graphics.isInsideCanvas(x, y)) {
			this._screenPressed = true;
			this._pressedTime = 0;
			if (event.touches.length >= 2) {
				this._onCancel(x, y);
			} else {
				this._onTrigger(x, y);
			}
			event.preventDefault();
		}
	}
	if (window.cordova || window.navigator.standalone) {
		event.preventDefault();
	}
};

TouchInput._onTouchMove = function(event) {
	let x, y;
	for (let touch of event.changedTouches) {
		if (Graphics._stretchWidth() > Graphics._stretchHeight()) {
			x = Graphics.pageToCanvasX(touch.pageX);
			y = Graphics.pageToCanvasY(touch.pageY);
		} else {
			y = Graphics._stretchWidth() / Graphics._realScale - Graphics.pageToCanvasX(touch.pageX);
			x = Graphics.pageToCanvasY(touch.pageY);
		}
		this._onMove(x, y);
	}
};

TouchInput._onTouchEnd = function(event) {
	let x, y;
	for (let touch of event.changedTouches) {
		if (Graphics._stretchWidth() > Graphics._stretchHeight()) {
			x = Graphics.pageToCanvasX(touch.pageX);
			y = Graphics.pageToCanvasY(touch.pageY);
		} else {
			y = Graphics._stretchWidth() / Graphics._realScale - Graphics.pageToCanvasX(touch.pageX);
			x = Graphics.pageToCanvasY(touch.pageY);
		}
		this._screenPressed = false;
		this._onRelease(x, y);
	}
};


/* =========================
	mobile UI	
========================= */

document.styleSheets[0].insertRule('#mobileControls {\
	position:absolute;\
	left:0px;\
	top:0px;\
	width:100%;\
	height:100%;\
	z-index:4;\
}', document.styleSheets[0].cssRules.length - 1);

document.styleSheets[0].insertRule('#button_X {\
	position:absolute;\
	right:20px;\
	bottom:170px;\
	width:0px;\
	height:0px;\
	background-image:url("../js/plugins/buttons/button_X.png");\
	background-size:100% 100%;\
	opacity:0.5;\
	border-radius:50% 50%;\
}', document.styleSheets[0].cssRules.length - 1);

document.styleSheets[0].insertRule('#button_Z {\
	position:absolute;\
	right:170px;\
	bottom:20px;\
	width:0px;\
	height:0px;\
	background-image:url("../js/plugins/buttons/button_Z.png");\
	background-size:100% 100%;\
	opacity:0.6;\
	border-radius:50% 50%;\
}', document.styleSheets[0].cssRules.length - 1);

document.styleSheets[0].insertRule('#button_shift {\
	position:absolute;\
	right:50px;\
	bottom:50px;\
	width:0px;\
	height:0px;\
	background-image:url("../js/plugins/buttons/button_shift.png");\
	background-size:100% 100%;\
	opacity:0.6;\
	border-radius:50% 50%;\
}', document.styleSheets[0].cssRules.length - 1);

document.styleSheets[0].insertRule('#shank_ZXS {\
	position:absolute;\
	left:30px;\
	bottom:30px;\
	width:0px;\
	height:0px;\
	background-image:url("../js/plugins/buttons/shank_ZXS.png");\
	background-size:100% 100%;\
	opacity:1.0;\
	border-radius:50% 50%;\
}', document.styleSheets[0].cssRules.length - 1);

document.styleSheets[0].insertRule('#shank_arrow_up {\
	transform:rotate(0deg);\
	-webkit-transform:rotate(0deg);\
	-moz-transform:rotate(0deg);\
	-ms-transform:rotate(0deg);\
	-o-transform:rotate(0deg);\
}', document.styleSheets[0].cssRules.length - 1);

document.styleSheets[0].insertRule('#shank_arrow_right {\
	transform:rotate(90deg);\
	-webkit-transform:rotate(90deg);\
	-moz-transform:rotate(90deg);\
	-ms-transform:rotate(90deg);\
	-o-transform:rotate(90deg);\
}', document.styleSheets[0].cssRules.length - 1);

document.styleSheets[0].insertRule('#shank_arrow_down {\
	transform:rotate(180deg);\
	-webkit-transform:rotate(180deg);\
	-moz-transform:rotate(180deg);\
	-ms-transform:rotate(180deg);\
	-o-transform:rotate(180deg);\
}', document.styleSheets[0].cssRules.length - 1);

document.styleSheets[0].insertRule('#shank_arrow_left {\
	transform:rotate(270deg);\
	-webkit-transform:rotate(270deg);\
	-moz-transform:rotate(270deg);\
	-ms-transform:rotate(270deg);\
	-o-transform:rotate(270deg);\
}', document.styleSheets[0].cssRules.length - 1);

document.styleSheets[0].insertRule('.shank_arrow {\
	position:absolute;\
	left:158px;\
	bottom:158px;\
	width:0px;\
	height:0px;\
	background-image:url("../js/plugins/buttons/shank_arrow.png");\
	background-size:100% 100%;\
	opacity:0.5;\
	transform-origin:50% 100%;\
}', document.styleSheets[0].cssRules.length - 1);


Graphics._createAllElements = function() {
	this._createErrorPrinter();
	this._createCanvas();
	this._createLoadingSpinner();
	this._createFPSCounter();
	this._createPanel();
};

Graphics._createPanel = function() {
	Graphics._panel = {};
	
	Graphics._panel._view = document.createElement("div");
	Graphics._panel._view.id = "mobileControls";
	document.body.appendChild(Graphics._panel._view);
	
	Graphics._panel._button_X = document.createElement("div");
	Graphics._panel._button_X.id = "button_X";
	Graphics._panel._button_X.bind_code = "88";
	Graphics._panel._view.appendChild(Graphics._panel._button_X);
	
	Graphics._panel._button_Z = document.createElement("div");
	Graphics._panel._button_Z.id = "button_Z";
	Graphics._panel._button_Z.bind_code = "90";
	Graphics._panel._view.appendChild(Graphics._panel._button_Z);
	
	Graphics._panel._button_shift = document.createElement("div");
	Graphics._panel._button_shift.id = "button_shift";
	Graphics._panel._button_shift.bind_code = "16";
	Graphics._panel._view.appendChild(Graphics._panel._button_shift);
	
	Graphics._panel._shank_ZXS = document.createElement("div");
	Graphics._panel._shank_ZXS.id = "shank_ZXS";
	Graphics._panel._view.appendChild(Graphics._panel._shank_ZXS);
	
	Graphics._panel._shank_ZXS_up = document.createElement("div");
	Graphics._panel._shank_ZXS_up.id = "shank_arrow_up";
	Graphics._panel._shank_ZXS_up.className = "shank_arrow";
	Graphics._panel._shank_ZXS_up.bind_code = "38";
	Graphics._panel._shank_ZXS.appendChild(Graphics._panel._shank_ZXS_up);
	
	Graphics._panel._shank_ZXS_right = document.createElement("div");
	Graphics._panel._shank_ZXS_right.id = "shank_arrow_right";
	Graphics._panel._shank_ZXS_right.className = "shank_arrow";
	Graphics._panel._shank_ZXS_right.bind_code = "39";
	Graphics._panel._shank_ZXS.appendChild(Graphics._panel._shank_ZXS_right);
	
	Graphics._panel._shank_ZXS_down = document.createElement("div");
	Graphics._panel._shank_ZXS_down.id = "shank_arrow_down";
	Graphics._panel._shank_ZXS_down.className = "shank_arrow";
	Graphics._panel._shank_ZXS_down.bind_code = "40";
	Graphics._panel._shank_ZXS.appendChild(Graphics._panel._shank_ZXS_down);
	
	Graphics._panel._shank_ZXS_left = document.createElement("div");
	Graphics._panel._shank_ZXS_left.id = "shank_arrow_left";
	Graphics._panel._shank_ZXS_left.className = "shank_arrow";
	Graphics._panel._shank_ZXS_left.bind_code = "37";
	Graphics._panel._shank_ZXS.appendChild(Graphics._panel._shank_ZXS_left);
	
	Graphics._updatePanel();
	
	Graphics._panel._button_X.addEventListener("touchstart", MobilePanel.button_press);
	Graphics._panel._button_Z.addEventListener("touchstart", MobilePanel.button_press);
	Graphics._panel._button_shift.addEventListener("touchstart", MobilePanel.button_press);
	
	Graphics._panel._button_X.addEventListener("touchmove", MobilePanel.button_solve);
	Graphics._panel._button_Z.addEventListener("touchmove", MobilePanel.button_solve);
	Graphics._panel._button_shift.addEventListener("touchmove", MobilePanel.button_solve);
	
	Graphics._panel._button_X.addEventListener("touchend", MobilePanel.button_release);
	Graphics._panel._button_Z.addEventListener("touchend", MobilePanel.button_release);
	Graphics._panel._button_shift.addEventListener("touchend", MobilePanel.button_release);
	
	Graphics._panel._button_X.addEventListener("touchcancel", MobilePanel.button_release);
	Graphics._panel._button_Z.addEventListener("touchcancel", MobilePanel.button_release);
	Graphics._panel._button_shift.addEventListener("touchcancel", MobilePanel.button_release);
	
	Graphics._panel._shank_ZXS.addEventListener("touchstart", MobilePanel.shank_press);
	Graphics._panel._shank_ZXS.addEventListener("touchmove", MobilePanel.shank_solve);
	Graphics._panel._shank_ZXS.addEventListener("touchend", MobilePanel.shank_release);
	Graphics._panel._shank_ZXS.addEventListener("touchcancel", MobilePanel.shank_release);
}

Graphics._updatePanel = function() {
	var button_pos_x, button_pos_y, button_radius;
	var scale;
	
	if (this._realScale < 0.6) {
		scale = 0.6;
	} else {
		scale = this._realScale;
	}
	
	// button X
	
	button_radius = 128 * scale;
	button_pos_x = 20 * scale;
	button_pos_y = 170 * scale;
	
	Graphics._panel._button_X.style.width = 
	Graphics._panel._button_X.style.height = button_radius + "px";
	
	Graphics._panel._button_X.style.right = button_pos_x + "px";
	Graphics._panel._button_X.style.bottom = button_pos_y + "px";
	
	// button Z
	
	button_radius = 128 * scale;
	button_pos_x = 170 * scale;
	button_pos_y = 20 * scale;
	
	Graphics._panel._button_Z.style.width = 
	Graphics._panel._button_Z.style.height = button_radius + "px";
	
	Graphics._panel._button_Z.style.right = button_pos_x + "px";
	Graphics._panel._button_Z.style.bottom = button_pos_y + "px";
	
	// button shift
	
	button_radius = 96 * scale;
	button_pos_x = 50 * scale;
	button_pos_y = 50 * scale;
	
	Graphics._panel._button_shift.style.width = 
	Graphics._panel._button_shift.style.height = button_radius + "px";
	
	Graphics._panel._button_shift.style.right = button_pos_x + "px";
	Graphics._panel._button_shift.style.bottom = button_pos_y + "px";
	
	// shank ZXS
	
	button_radius = 256 * scale;
	button_pos_x = 30 * scale;
	button_pos_y = 30 * scale;
	
	Graphics._panel._shank_ZXS.style.width = 
	Graphics._panel._shank_ZXS.style.height = button_radius + "px";
	
	Graphics._panel._shank_ZXS.style.left = button_pos_x + "px";
	Graphics._panel._shank_ZXS.style.bottom = button_pos_y + "px";
	
	// shank arrows
	
	button_radius = 144.5 * scale;
	button_pos_x = 56.5 * scale;
	button_pos_y = 127 * scale;
	
	Graphics._panel._shank_ZXS_up.style.width = 
	Graphics._panel._shank_ZXS_right.style.width = 
	Graphics._panel._shank_ZXS_down.style.width = 
	Graphics._panel._shank_ZXS_left.style.width = button_radius + "px";
	
	Graphics._panel._shank_ZXS_up.style.height = 
	Graphics._panel._shank_ZXS_right.style.height = 
	Graphics._panel._shank_ZXS_down.style.height = 
	Graphics._panel._shank_ZXS_left.style.height = button_radius * 0.7 + "px";
	
	Graphics._panel._shank_ZXS_up.style.left = button_pos_x + "px";
	Graphics._panel._shank_ZXS_right.style.left = (button_pos_x + 2) + "px";
	Graphics._panel._shank_ZXS_down.style.left = button_pos_x + "px";
	Graphics._panel._shank_ZXS_left.style.left = (button_pos_x - 2) + "px";
	
	Graphics._panel._shank_ZXS_up.style.bottom = (button_pos_y + 2) + "px";
	Graphics._panel._shank_ZXS_right.style.bottom = button_pos_y + "px";
	Graphics._panel._shank_ZXS_down.style.bottom = (button_pos_y - 2) + "px";
	Graphics._panel._shank_ZXS_left.style.bottom = button_pos_y + "px";
};


function MobilePanel(){
	throw new Error("This is a static class");
}

MobilePanel.simulateKeyElement = null;

MobilePanel.simulatePressed = function() {
	return true;
}

MobilePanel.button_press = function(event) {
	var key_event;
	
	event.stopPropagation();
	event.preventDefault();
	
	key_event = {
		keyCode:parseInt(this.bind_code),
		preventDefault:MobilePanel.simulatePressed
	};
	
	this.style.opacity = "1.0";
	Input._onKeyDown(key_event);
}

MobilePanel.button_solve = function(event) {
	event.stopPropagation();
	event.preventDefault();
}

MobilePanel.button_release = function(event) {
	var key_event;
	
	event.stopPropagation();
	event.preventDefault();
	
	key_event = {
		keyCode:parseInt(this.bind_code),
		preventDefault:MobilePanel.simulatePressed
	};
	
	this.style.opacity = "0.5";
	Input._onKeyUp(key_event);
}

MobilePanel.shank_press = function() {
	var x, y, rx, ry, tan0;
	var key_event, el;
	
	event.stopPropagation();
	event.preventDefault();
	
	x = event.changedTouches[0].pageX;
	y = event.changedTouches[0].pageY;
	
	if (x < 0 || y < 0) {
		return false;
	}
	
	el = MobilePanel.shank_get_arrow(x, y, this);
	MobilePanel.put_simulate_key(el);
}

MobilePanel.shank_solve = function() {
	var x, y, rx, ry, tan0;
	var key_event, el;
	
	event.stopPropagation();
	event.preventDefault();
	
	x = event.changedTouches[0].pageX;
	y = event.changedTouches[0].pageY;
	
	if (x < 0 || y < 0) {
		return false;
	}
	
	el = MobilePanel.shank_get_arrow(x, y, this);
	
	if (MobilePanel.simulateKeyElement != el) {
		MobilePanel.put_simulate_key(el);
	}
}

MobilePanel.shank_release = function() {
	var key_event;
	
	event.stopPropagation();
	event.preventDefault();
	
	MobilePanel.put_simulate_key(null);
}

MobilePanel.shank_get_arrow = function(x, y, el) {
	var rx, ry, tan0;
	
	if (Graphics._stretchWidth() > Graphics._stretchHeight()) {
		rx = x - (el.offsetLeft + el.offsetWidth / 2);
		ry = y - (el.offsetTop + el.offsetHeight / 2);
	} else {
		rx = y - (el.offsetLeft + el.offsetWidth / 2);
		ry = (Graphics._stretchWidth() - x) - (el.offsetTop + el.offsetHeight / 2);
	}
	
	if (rx > 0) {
		tan0 = ry / rx;
		
		if (tan0 > 1) {
			return Graphics._panel._shank_ZXS_down;
		} else if (tan0 > -1) {
			return Graphics._panel._shank_ZXS_right;
		} else {
			return Graphics._panel._shank_ZXS_up;
		}
	} else if (rx < 0) {
		tan0 = ry / rx;
		
		if (tan0 < -1) {
			return Graphics._panel._shank_ZXS_down;
		} else if (tan0 < 1) {
			return Graphics._panel._shank_ZXS_left;
		} else {
			return Graphics._panel._shank_ZXS_up;
		}
	} else {
		if (ry > 0) {
			return Graphics._panel._shank_ZXS_down;
		} else {
			return Graphics._panel._shank_ZXS_up;
		}
	}
}

MobilePanel.put_simulate_key = function(el) {
	if (MobilePanel.simulateKeyElement) {
		key_event = {
			keyCode:parseInt(MobilePanel.simulateKeyElement.bind_code),
			preventDefault:MobilePanel.simulatePressed
		};
		
		MobilePanel.simulateKeyElement.style.opacity = "0.5";
		Input._onKeyUp(key_event);
		
	}
	
	if (el) {
		key_event = {
			keyCode:parseInt(el.bind_code),
			preventDefault:MobilePanel.simulatePressed
		};
		
		el.style.opacity = "1.0";
		Input._onKeyDown(key_event);
		MobilePanel.simulateKeyElement = el;
	} else {
		MobilePanel.simulateKeyElement = null;
	}
}


/* =========================
	full screen mode	
========================= */

document.body.ontouchend = function(event) {
	event.stopPropagation();
	event.preventDefault();
	
	if(!Graphics._isFullScreen()) {
		//window.resizeTo(screen.availWidth,screen.availHeight);
		//window.moveTo(0,0);
		Graphics._requestFullScreen();
	}
	
	this.ontouchend = null;
}